/******************************************************************************/
/* LINKER COMMAND FILE FOR MSPBoot BOOTLOADER USING msp430f5529  */
/* File generated with MSPBootLinkerGen.pl on 06-22-2017 */
/*----------------------------------------------------------------------------*/


/****************************************************************************/
/* SPECIFY THE SYSTEM MEMORY MAP                                            */
/****************************************************************************/
/* The following definitions can be changed to customize the memory map for a different device
 *   or other adjustments
 *  Note that the changes should match the definitions used in MEMORY and SECTIONS
 *
 */
/* RAM Memory Addresses */
__RAM_Start = 0x2400;                 /* RAM Start */
__RAM_End = 0x43FF;                     /* RAM End */
    /* RAM shared between App and Bootloader, must be reserved */
    PassWd = 0x2400;                 /* Password sent by App to force boot  mode */
    StatCtrl = 0x2402;             /* Status and Control  byte used by Comm */
    CI_State_Machine = 0x2403;         /*  State machine variable used by Comm */
    CI_Callback_ptr = 0x2404;   /* Pointer to Comm callback structure */
    /* Unreserved RAM used for Bootloader or App purposes */
    _NonReserved_RAM_Start = 0x2408; /* Non-reserved RAM */

/* Flash memory addresses */
/* App area     : 4400-EBFF & 10000-14DFF*/
/* Download area: 14E00-243FF*/
/* Boot area    : EC00-FFFF*/

__Flash_Start = 0x4400;             /* Start of Application area */
   /* Reserved Flash locations for Bootloader Area */
    __Boot_Start = 0xEC00;         /* Boot flash */
    __Boot_Reset = 0xFFFE;                          /* Boot reset vector */
    __Boot_VectorTable = 0xFF80;      /* Boot vector table */
    __Boot_SharedCallbacks_Len = 6; /* Length of shared callbacks (2 calls =4B(msp430) or 8B(msp430x) */
    __Boot_SharedCallbacks = 0xFF7A; /* Start of Shared callbacks */
     _BOOT_APPVECTOR = __Boot_SharedCallbacks;       /* Definition for application table             */
    _Appl_Proxy_Vector_Start = 0xEB02; /* Proxy interrupt table */
    /* Reserved Flash locations for Application Area */
 
/* MEMORY definition, adjust based on definitions above */
MEMORY
{
    SFR                     : origin = 0x0000, length = 0x0010
    PERIPHERALS_8BIT        : origin = 0x0010, length = 0x00F0
    PERIPHERALS_16BIT       : origin = 0x0100, length = 0x0100
    // RAM from _NonReserved_RAM_Start - __RAM_End
    RAM                     : origin = 0x2408, length = 0x1FF8
    USBRAM                  : origin = 0x1C00, length = 0x800
	// Info Mem Sections
	INFOA                   : origin = 0x1980, length = 0x80
    INFOB                   : origin = 0x1900, length = 0x80
    INFOC                   : origin = 0x1880, length = 0x80
    INFOD                   : origin = 0x1800, length = 0x80
    // Flash from _App_Start -> (APP_PROXY_VECTORS-1)
    FLASH                   : origin = 0x4402, length = 0xA700
    FLASH2                  : origin = 0x10000, length = 0x4E00
    // Interrupt Proxy table from  _App_Proxy_Vector_Start->(RESET-1)
    APP_PROXY_VECTORS       : origin = 0xEB02, length = 252
    // App reset from _App_Reset_Vector
    RESET                   : origin = 0xEBFE, length = 0x0002
}

/****************************************************************************/
/* SPECIFY THE SECTIONS ALLOCATION INTO MEMORY                              */
/****************************************************************************/

SECTIONS
{
    .bss        : {} > RAM                /* GLOBAL & STATIC VARS              */
    .data       : {} > RAM                /* GLOBAL & STATIC VARS              */
    .sysmem     : {} > RAM                /* DYNAMIC MEMORY ALLOCATION AREA    */
    .stack      : {} > RAM (HIGH)         /* SOFTWARE SYSTEM STACK             */

    .text:_isr  : {}  > FLASH            /* Code ISRs                         */
    #ifndef __LARGE_CODE_MODEL__
        .text       : {} >> FLASH               /* CODE                 */
    #else 
        .text       : {} >> FLASH | FLASH2      /* CODE                 */ 
    #endif 

        .cinit      : {} > FLASH        /* INITIALIZATION TABLES*/ 
    #ifndef __LARGE_DATA_MODEL__ 
        .const      : {} >> FLASH       /* CONSTANT DATA        */ 
    #else 
        .const      : {} >> FLASH2 | FLASH    /* CONSTANT DATA        */ 
    #endif 

    .cio        : {} > RAM                /* C I/O BUFFER                      */
	
	.infoA     : {} > INFOA              /* MSP430 INFO FLASH Memory segments */
    .infoB     : {} > INFOB
    .infoC     : {} > INFOC
    .infoD     : {} > INFOD

    .APP_PROXY_VECTORS : {} > APP_PROXY_VECTORS /* INTERRUPT PROXY TABLE            */
    .reset       : {}               > RESET  /* MSP430 RESET VECTOR                 */
}

/****************************************************************************/
/* INCLUDE PERIPHERALS MEMORY MAP                                           */
/****************************************************************************/

-l msp430f5529.cmd

