/*
 * \file   main.c
 *
 * \brief  Sample application template for MSPBoot 
*/
/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
 
 /* Includes */
#include "msp430.h"
#include <stdint.h>
#include "TI_MSPBoot_Mgr_CI_eUSCI_UART.h"
#include "TI_MSPBoot_Mgr_Vectors.h"
//TODO: Add application include files

/* Constant Definitions */
//TODO: Add application #define(s)

/* Global Variable Definitions*/
static uint8_t RxByte;     /*! Byte received from comm interface */
static uint8_t new_data;   /*! Flag to indicate new data reception */
//TODO: Add global variable defintions

/* Global Function Protoypes */
extern void TI_MSPBoot_JumpToBoot( void );  // Use to jump to the bootloader
extern void TI_MSPBoot_Comm_Init(  void (*RxCallback)(uint8_t),
                                   void (*TxCallback)(uint8_t *),
                                   void (*ErrorCallback)(uint8_t));

/* Local Function Prototypes */
#ifdef SHARED_PI
static void App_Comm_Rx(uint8_t data);
#endif
//TODO: Add local function prototypes

/* Main */
int main( void )
{
  // Stop watchdog timer to prevent time out reset
  WDTCTL = WDTPW + WDTHOLD;
  
#if (defined(SHARED_PI))
	/* If the MSPBoot supports it, initialize the shared comm interface
		Note that this function will fail if the comm is not shared and the init
		vector is invalid */
	// This example only uses the RX callbacks
	// The TX and error callback are not implemented
	// I2C callbacks are disabled
    TI_MSPBoot_Comm_Init(App_Comm_Rx, NULL, NULL);
#endif

    new_data = 0;
    __enable_interrupt();

    while (1)
    {
		// TI_MSPBoot_Comm_Isr wakes device from LPM0
        __bis_SR_register(LPM0_bits + GIE);
        __disable_interrupt();
		
		if(new_data == 1)
		{
			__disable_interrupt();
			switch (RxByte)
            {
                // Process byte received from Shared Comm
                case 0xAA: //TODO: change value for your application
                    // Force Boot mode
                    TI_MSPBoot_JumpToBoot();
                break;
                default:
				    __no_operation();
                break;
            }
			
			new_data = 0; // Clear flag
		}
		
		//TODO: enter application code here

    }
}

//TODO: Add ISR and function code here

#ifdef SHARED_PI
/******************************************************************************
 *
 * @brief   RXCallback for Shared Comm interface
 *  This function demonstrates how to receive data from the shared Communication
 *  interface. Function is called by MSPBoot when a byte is received from the host
 *
 *   Note that this function prototype should be accessible by TI_MSPBoot_Mgr_PI.c
 *   and is called by TI_MSPBoot_Comm_Init
 *
 * @return  none
 *****************************************************************************/
static void App_Comm_Rx(uint8_t data)
{
    RxByte = data;
    new_data = 1;
}

#endif //SHARED_PI

/******************************************************************************
 *
 * @brief   Dummy Interrupt service routine
 *  This ISR will be executed if an undeclared interrupt is called
 *
 * @return  none
 *****************************************************************************/
#pragma vector = unused_interrupts
__interrupt void Dummy_Isr(void)
{
    while(1)
        ;
}
